import { CommonModule } from '@angular/common';
import { Component, ChangeDetectionStrategy, ViewEncapsulation, ChangeDetectorRef, Optional, Self, Input, NgModule } from '@angular/core';
import { mixinDisabled, mixinErrorState, ErrorStateMatcher, DateAdapter, MatNativeDateModule } from '@angular/material/core';
import { SimpleSelectModule } from '@nutrify/ngx-simple-select';
export { SimpleSelectModule } from '@nutrify/ngx-simple-select';
import { Subscription } from 'rxjs';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { Validators, NgForm, FormGroupDirective, NgControl } from '@angular/forms';

let nextUniqueId = 0;
class SimpleDatepickerBase {
    constructor(_defaultErrorStateMatcher, _parentForm, _parentFormGroup, ngControl) {
        this._defaultErrorStateMatcher = _defaultErrorStateMatcher;
        this._parentForm = _parentForm;
        this._parentFormGroup = _parentFormGroup;
        this.ngControl = ngControl;
    }
}
// eslint-disable-next-line @typescript-eslint/naming-convention
const _SimpleDatepickerMixinBase = mixinDisabled(mixinErrorState(SimpleDatepickerBase));
class DatepickerComponent extends _SimpleDatepickerMixinBase {
    constructor(cdRef, _defaultErrorStateMatcher, _parentForm, _parentFormGroup, ngControl, dateAdapter) {
        super(_defaultErrorStateMatcher, _parentForm, _parentFormGroup, ngControl);
        this.cdRef = cdRef;
        this._defaultErrorStateMatcher = _defaultErrorStateMatcher;
        this.ngControl = ngControl;
        this.dateAdapter = dateAdapter;
        this.dayPlaceholder = 'Day';
        this.monthPlaceholder = 'Month';
        this.yearPlaceholder = 'Year';
        this.months = [
            'January',
            'February',
            'March',
            'April',
            'May',
            'June',
            'July',
            'August',
            'September',
            'October',
            'November',
            'December'
        ];
        this.numberOfDays = 31;
        this.selectedYear = null;
        this.selectedMonth = null;
        this.validationError = false;
        this._value = null;
        this._required = false;
        this._errorState = false;
        this._selectedDay = null;
        this.previousDay = null;
        this.localeSubscription = Subscription.EMPTY;
        this.uid = `simple-datepicker-${++nextUniqueId}`;
        this.validators = new Map([
            ['simpleDatepickerMin', (control) => {
                    const controlValue = this.coerceDateProperty(this.dateAdapter.deserialize(control.value));
                    return (!this.min || !controlValue ||
                        this.dateAdapter.compareDate(this.min, controlValue) <= 0) ?
                        null : { simpleDatepickerMin: { min: this.min, actual: controlValue } };
                }
            ],
            ['simpleDatepickerMax', (control) => {
                    const controlValue = this.coerceDateProperty(this.dateAdapter.deserialize(control.value));
                    return (!this.max || !controlValue ||
                        this.dateAdapter.compareDate(this.max, controlValue) >= 0) ?
                        null : { simpleDatepickerMax: { max: this.max, actual: controlValue } };
                }
            ]
        ]);
        this.propagateChange = (_) => { };
        this.propagateTouched = () => { };
        this.validatorOnChange = () => { };
        /* eslint-disable @typescript-eslint/member-ordering */
        /** The form control validator for the min date. */
        this.minValidator = this.validators.get('simpleDatepickerMin');
        /** The form control validator for the max date. */
        this.maxValidator = this.validators.get('simpleDatepickerMax');
        this.validator = Validators.compose([this.minValidator, this.maxValidator]);
        if (this.ngControl) {
            // Note: we provide the value accessor through here, instead of
            // the `providers` to avoid running into a circular import.
            this.ngControl.valueAccessor = this;
            this.ngControl.control.setValidators(this.validator);
            this.ngControl.control.updateValueAndValidity();
        }
        // Force setter to be called in case id was not specified.
        this.id = this.id;
        // Update the displayed date when the locale changes.
        this.localeSubscription = dateAdapter.localeChanges.subscribe(() => {
            this.value = this.value;
        });
    }
    get id() { return this._id; }
    set id(id) {
        this._id = id || this.uid;
    }
    get value() { return this._value; }
    set value(value) {
        this._value = this.coerceDateProperty(value);
    }
    get required() { return this._required; }
    set required(required) {
        this._required = coerceBooleanProperty(required);
    }
    get min() { return this._min; }
    set min(min) {
        this._min = this.coerceDateProperty(this.dateAdapter.deserialize(min));
        this.validatorOnChange();
    }
    get max() { return this._max; }
    set max(max) {
        this._max = this.coerceDateProperty(this.dateAdapter.deserialize(max));
        this.validatorOnChange();
    }
    get errorState() {
        return this._errorState;
    }
    set errorState(errorState) {
        this._errorState = errorState;
    }
    get selectedDay() {
        if (!this._selectedDay && this.previousDay <= this.numberOfDays) {
            this._selectedDay = this.previousDay;
        }
        return this._selectedDay;
    }
    set selectedDay(selectedDay) {
        this._selectedDay = selectedDay;
    }
    get startingYear() {
        return this.min ? this.min.getFullYear() : new Date().getFullYear() - 115;
    }
    get endingYear() {
        return this.max ? this.max.getFullYear() : new Date().getFullYear() - 16;
    }
    coerceDateProperty(obj) {
        return (this.dateAdapter.isDateInstance(obj) && this.dateAdapter.isValid(obj)) ? obj : null;
    }
    daysInMonth(month, year = 0) {
        switch (month) {
            case 2:
                const isLeapYear = ((year % 4 === 0) && (year % 100 !== 0)) || (year % 400 === 0);
                this.numberOfDays = isLeapYear ? 29 : 28;
                break;
            case 4:
            case 6:
            case 9:
            case 11:
                this.numberOfDays = 30;
                break;
            default:
                this.numberOfDays = 31;
        }
        return this.numberOfDays;
    }
    updateDay(day) {
        this.selectedDay = day;
        if (day) {
            this.previousDay = day;
        }
        this.propagateTouched();
        this.emit();
    }
    update() {
        this.numberOfDays = this.daysInMonth(this.selectedMonth, this.selectedYear);
        this.propagateTouched();
        this.cdRef.detectChanges();
        this.emit();
    }
    emit() {
        if (this.selectedDay !== null && this.selectedMonth !== null && this.selectedYear !== null) {
            if (this.coerceDateProperty(this.value)) {
                this.value.setDate(this.selectedDay);
                this.value.setMonth(this.selectedMonth - 1);
                this.value.setFullYear(this.selectedYear);
            }
            else {
                this.value = new Date(this.selectedYear, this.selectedMonth - 1, this.selectedDay);
            }
        }
        else {
            this.value = null;
        }
        this.propagateChange(this.value);
    }
    validate(c) {
        return this.validator ? this.validator(c) : null;
    }
    registerOnValidatorChange(fn) {
        this.validatorOnChange = fn;
    }
    ngDoCheck() {
        if (this.ngControl) {
            this.updateErrorState();
            if (this.ngControl.errors && !this.ngControl.errors.required) {
                this.validationError = true;
            }
            else {
                this.validationError = false;
            }
        }
    }
    writeValue(value) {
        if (this.coerceDateProperty(value)) {
            this.value = value;
            this.selectedDay = value.getDate();
            this.selectedMonth = value.getMonth();
            this.selectedYear = value.getFullYear();
            this.cdRef.detectChanges();
        }
    }
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    registerOnTouched(fn) {
        this.propagateTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    ngOnDestroy() {
        this.localeSubscription.unsubscribe();
        this.stateChanges.complete();
    }
}
DatepickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'simple-datepicker',
                exportAs: 'simpleDatepicker',
                template: "<simple-select [ngModel]=\"selectedDay\" (ngModelChange)=\"updateDay($event)\" [placeholder]=\"dayPlaceholder\" class=\"simple-datepicker-day\" [disabled]=\"disabled\" [required]=\"required\">\r\n  <simple-option *ngFor=\"let x of [].constructor(numberOfDays); let i = index\" [value]=\"i+1\">{{i+1}}</simple-option>\r\n</simple-select>\r\n<simple-select [(ngModel)]=\"selectedMonth\" (ngModelChange)=\"update()\" [placeholder]=\"monthPlaceholder\" class=\"simple-datepicker-month\" [disabled]=\"disabled\" [required]=\"required\">\r\n  <simple-option *ngFor=\"let x of [].constructor(12); let i = index\" [value]=\"i+1\">{{months[i]}}</simple-option>\r\n</simple-select>\r\n<simple-select [(ngModel)]=\"selectedYear\" (ngModelChange)=\"update()\" [placeholder]=\"yearPlaceholder\" class=\"simple-datepicker-year\" [disabled]=\"disabled\" [required]=\"required\">\r\n  <simple-option *ngFor=\"let x of [].constructor(endingYear - startingYear + 1); let i = index\" [value]=\"endingYear-i\">{{endingYear-i}}</simple-option>\r\n</simple-select>\r\n",
                inputs: ['disabled'],
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                // eslint-disable-next-line
                host: {
                    '[attr.id]': 'id',
                    class: 'simple-datepicker',
                    '[attr.min]': 'min ? dateAdapter.toIso8601(min) : null',
                    '[attr.max]': 'max ? dateAdapter.toIso8601(max) : null',
                    '[attr.aria-disabled]': 'disabled.toString()',
                    '[attr.aria-invalid]': 'errorState',
                    '[attr.aria-required]': 'required.toString()',
                    '[class.simple-datepicker-disabled]': 'disabled',
                    '[class.simple-datepicker-invalid]': 'validationError'
                }
            },] }
];
DatepickerComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: ErrorStateMatcher },
    { type: NgForm, decorators: [{ type: Optional }] },
    { type: FormGroupDirective, decorators: [{ type: Optional }] },
    { type: NgControl, decorators: [{ type: Self }, { type: Optional }] },
    { type: DateAdapter, decorators: [{ type: Optional }] }
];
DatepickerComponent.propDecorators = {
    dayPlaceholder: [{ type: Input }],
    monthPlaceholder: [{ type: Input }],
    yearPlaceholder: [{ type: Input }],
    months: [{ type: Input }],
    id: [{ type: Input }],
    value: [{ type: Input }],
    required: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }]
};

class SimpleDatepickerModule {
}
SimpleDatepickerModule.decorators = [
    { type: NgModule, args: [{
                declarations: [DatepickerComponent],
                imports: [
                    CommonModule,
                    SimpleSelectModule,
                    MatNativeDateModule
                ],
                exports: [DatepickerComponent, SimpleSelectModule]
            },] }
];

/*
 * Public API Surface of simple-datepicker
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DatepickerComponent, SimpleDatepickerModule };
//# sourceMappingURL=nutrify-ngx-simple-datepicker.js.map
